#!/bin/sh

CLUSTERDIR='/data/rubiscluster'
CHECKPOINTDIR='/data/rubiscluster/checkpointfolder'
IMGFILE='/data/rubisva/rubisva_1.0.img'
RUBISCONFIGFILE='/etc/xen/rubisva'
MACPREFIX='00:16:3e:41:29:4'
TAKESNAPSHOT=0

function usage()
{
	echo -e Usage: $0 scenario_number \
	'\n' available scenario_number are: \
	'\n' 1: deploy 1 client - 1 apache - 2 jboss servers - 1 db \
	'\n' 2: deploy 2 clients - 1 apache - 2 jboss servers - 1 db \
	'\n' 3: deploy 2 clients - 2 apaches - 2 jboss servers - 1 db 
	exit 0;
}

function inform_about_space()
{
	echo -e the cluster deployment is about to start \
	'\n' note it may take several minutes \
	'\n' make sure you have $1 GB in the /data partition. \
	'\n' Do you want to proceed ? .
}

function ping_subnet()
{
	j=$2

	while [ $j -gt $1 ]
	do
		ping -c1 192.168.1.$j > /dev/null
		let "j=j - 1" 
	done
}

function get_ip_from_arpline()
{
	
	ip=` echo $2 | sed s/\(// | sed s/\)//`
}

function take_snapshot()
{
	#take a snapshot of the cluster
	#we start by clients
	inc=`expr $1 - 1`
	while [ $inc -gt 0 ]
	do
		#xm pause rubisva_$inc 
		xm save rubisva_$inc $CHECKPOINTDIR'/'rubisva_$inc
		let "inc=inc-1"
	done
}


function start_cross_config()
{
	# $1 : number of domU
	# $2, $3 , $4 , $5 : successively the number of db jboss apache and client 
	
	i=0
	
	CONFIG=$2' '$3' '$4' '$5
	
	#this will help to use the ip adress from the mac adresses 
	ping_subnet 90 100

	#get the ip from mac
	while [ $i -lt $1 ]
	do
		#get ip line
		ip=` arp -a |grep -i $MACPREFIX$i  `
		#get the ip from mac address
		get_ip_from_arpline $ip
		
		CONFIG=$CONFIG' '$ip			
		#now that ips are append we need to assign a role to each ip using the distribution  specified by the first 4 number of the chain
		let "i=i + 1"
		# this will copy the finger print of the domU into /root/.ssh/known_hosts remember if there is an expiration or a problem about this you will need to accept ont more time manually and change the code so he will changer the finger print of all host in the subnet.
		#echo $ip
		./check_finger_print.sh $ip
	done

	#do config 
	./do_cross_config.sh $CONFIG
}
function runlevel1()
{
		
	echo -e run level1: copying image into $CLUSTERDIR...'\n'
       if [ ! -d $CLUSTERDIR ]; then
                mkdir $CLUSTERDIR
        else
			echo do you want keep existing images in $CLUSTERDIR 
			select STARTCLEAN in yes no stop
			do 
				if [ "$STARTCLEAN" == "yes" ];then
					break
				elif [ "$STARTCLEAN" == "no"  ];then
					echo cleaning up images in $CLUSTERDIR ....
					rm -f $CLUSTERDIR/rubisva_*
					break
				elif [ "$STARTCLEAN" == "stop" ];then
					exit 0
				else
					continue
			fi
			done
	   fi
	
        i=0
        while [ $i -lt $1 ]
        do
			
			if [ ! -f $CLUSTERDIR/rubisva_$i.img ];then
			 echo copying images rubisva_$i.img  in $CLUSTERDIR ....
                cp -f $IMGFILE $CLUSTERDIR/rubisva_$i.img
			fi
      		let "i=i + 1"
	   done 	

}

function runlevel2()
{
	echo -e run level2: creating xen config_file in /etc/xen/ ...'\n'
        
	   i=0
        while [ $i -lt $1 ]
        do
                cp -f $RUBISCONFIGFILE /etc/xen/rubisva_$i
      		let "i=i + 1"
	   done 	 
	  
        i=0
        while [ $i -lt $1 ]
        do
			
			echo configuring xen config_file $i in $CLUSTERDIR ....
			
			#setting rubis name
                sed  -i.old -e s/\"rubisva\"/\"rubisva_$i\"/ /etc/xen/rubisva_$i
			
			#setting image file
			IMGFILE_SED=`echo $IMGFILE | sed -e s#\/#"\\\\\/"#g `
			CLUSTERDIR_SED=`echo $CLUSTERDIR/rubisva_$i.img | sed -e s#\/#"\\\\\/"#g `
			echo $IMGFILE_SED
			echo $CLUSTERDIR_SED
			sed  -i.old -e 's#'$IMGFILE_SED'#'$CLUSTERDIR_SED'#g' /etc/xen/rubisva_$i

			#setting mac address to avoid the dhcp giving the same adress to all domU

			linetoreplace=`cat  /etc/xen/rubisva_$i | grep -n 'vif =' | grep -m 1 -o '^[0-9]*' `
			sed  -i.old -e "$linetoreplace c\vif = [ 'mac=$MACPREFIX$i' ]" /etc/xen/rubisva_$i
			
      		let "i=i + 1"
	   done 	
	 
	   
	
	
}

function runlevel3()
{
	echo -e run level2: starting all rubis va. info: $1 va will be started ...'\n'
        i=0
        while [ $i -lt $1 ]
        do
			echo starting domUs rubisva_$i.img ...
                 xm create rubisva_$i
      		let "i=i + 1"
	   done 	
	# wait almost 3 min for all servers to be started
     sleep 200 
	
}

if [ $# = 0 ];then
	usage
	exit 1
fi

#create all required dir if need
mkdir -p $CLUSTERDIR
mkdir -p $CHECKPOINTDIR


while getopts s opt
do	case "$opt" in
	s)	TAKESNAPSHOT=1 
		shift;;

	esac
done


case $1 in 

1) 
	
	NUMBEROFVA=5
	# in the following 1 2 1 1 means there is  (from the left to the right), 1 db  - 2 jboss - 1 apache - 1 client 
	ALLIP='1 2 1 1' 
	inform_about_space 50
;;
2)  
	
	NUMBEROFVA=6
	# in the following 1 2 1 1 means there is  (from the left to the right), 1 db -  2 jboss - 1 apache - 2 clients 
	ALLIP='1 2 1 2' 
	inform_about_space 60
;;

3)  
	
	NUMBEROFVA=7
	# in the following 1 2 1 1 means there is  (from the left to the right), 1 db -  2 jboss - 2 apaches - 2 clients 
	ALLIP='1 2 2 2' 
	inform_about_space 70
;;
	

*) echo you have entered $1 
   usage
;;

esac

select CHOICE in yes no
	do
		if [ "$CHOICE" == "yes" ]; then
			runlevel1 $NUMBEROFVA 
			runlevel2 $NUMBEROFVA 
			runlevel3 $NUMBEROFVA 
			#start the deployment procedure
			#we use the string ALLIP 
			start_cross_config $NUMBEROFVA $ALLIP
			[ $TAKESNAPSHOT == 1 ] && take_snapshot $NUMBEROFVA
			break
		elif [  "$CHOICE" == "no" ]; then
			exit 0
		else
			continue	
		fi
	done


